{{ vars.permissions|generate_import }}
# from invenio_records_permissions.generators import SystemProcess, AnyUser

class {{ vars.permissions|class_header }}:
    """{{ vars.record.class }} permissions.
        Values in this class will be merged with permission presets.
    """

    can_search = []
    can_read = []
    can_create = []
    can_update = []
    can_delete = []
    can_manage = []
    can_read_files = []
    can_update_files = []

{{ vars.permissions|extra_code }}